@@ -23,7 +23,6 @@ import warnings
 import zlib
 from collections import defaultdict
 from functools import partial
-from io import StringIO
 from random import Random
 from typing import (
     TYPE_CHECKING,
@@ -34,6 +33,8 @@ from typing import (
     Hashable,
     List,
     Optional,
+    Tuple,
+    Type,
     TypeVar,
     Union,
     overload,
@@ -74,10 +75,12 @@ from hypothesis.internal.compat import (
     int_from_bytes,
 )
 from hypothesis.internal.conjecture.data import ConjectureData, Status
-from hypothesis.internal.conjecture.engine import ConjectureRunner
+from hypothesis.internal.conjecture.engine import BUFFER_SIZE, ConjectureRunner
+from hypothesis.internal.conjecture.junkdrawer import ensure_free_stackframes
 from hypothesis.internal.conjecture.shrinker import sort_key
 from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.internal.escalation import (
+    current_pytest_item,
     escalate_hypothesis_internal_error,
     format_exception,
     get_interesting_origin,
@@ -92,10 +95,12 @@ from hypothesis.internal.reflection import (
     get_signature,
     impersonate,
     is_mock,
+    nicerepr,
     proxies,
     repr_call,
 )
 from hypothesis.internal.scrutineer import Tracer, explanatory_lines
+from hypothesis.internal.validation import check_type
 from hypothesis.reporting import (
     current_verbosity,
     report,
@@ -103,23 +108,21 @@ from hypothesis.reporting import (
     with_reporter,
 )
 from hypothesis.statistics import describe_targets, note_statistics
-from hypothesis.strategies._internal.collections import TupleStrategy
 from hypothesis.strategies._internal.misc import NOTHING
 from hypothesis.strategies._internal.strategies import (
     Ex,
-    MappedSearchStrategy,
     SearchStrategy,
+    check_strategy,
 )
-from hypothesis.utils.conventions import infer
 from hypothesis.vendor.pretty import RepresentationPrinter
 from hypothesis.version import __version__
 
-if sys.version_info >= (3, 10):  # pragma: no cover
-    from types import EllipsisType as InferType
+if sys.version_info >= (3, 10):
+    from types import EllipsisType as EllipsisType
 elif TYPE_CHECKING:
-    from builtins import ellipsis as InferType
-else:
-    InferType = type(Ellipsis)
+    from builtins import ellipsis as EllipsisType
+else:  # pragma: no cover
+    EllipsisType = type(Ellipsis)
 
 
 TestFunc = TypeVar("TestFunc", bound=Callable)
@@ -135,27 +138,130 @@ _hypothesis_global_random = None
 class Example:
     args = attr.ib()
     kwargs = attr.ib()
+    # Plus two optional arguments for .xfail()
+    raises = attr.ib(default=None)
+    reason = attr.ib(default=None)
 
 
-def example(*args: Any, **kwargs: Any) -> Callable[[TestFunc], TestFunc]:
+class example:
     """A decorator which ensures a specific example is always tested."""
-    if args and kwargs:
-        raise InvalidArgument(
-            "Cannot mix positional and keyword arguments for examples"
-        )
-    if not (args or kwargs):
-        raise InvalidArgument("An example must provide at least one argument")
 
-    hypothesis_explicit_examples: List[Example] = []
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        if args and kwargs:
+            raise InvalidArgument(
+                "Cannot mix positional and keyword arguments for examples"
+            )
+        if not (args or kwargs):
+            raise InvalidArgument("An example must provide at least one argument")
 
-    def accept(test):
+        self.hypothesis_explicit_examples: List[Example] = []
+        self._this_example = Example(tuple(args), kwargs)
+
+    def __call__(self, test: TestFunc) -> TestFunc:
         if not hasattr(test, "hypothesis_explicit_examples"):
-            test.hypothesis_explicit_examples = hypothesis_explicit_examples
-        test.hypothesis_explicit_examples.append(Example(tuple(args), kwargs))
+            test.hypothesis_explicit_examples = self.hypothesis_explicit_examples  # type: ignore
+        test.hypothesis_explicit_examples.append(self._this_example)  # type: ignore
         return test
 
-    accept.hypothesis_explicit_examples = hypothesis_explicit_examples  # type: ignore
-    return accept
+    def xfail(
+        self,
+        condition: bool = True,
+        *,
+        reason: str = "",
+        raises: Union[
+            Type[BaseException], Tuple[Type[BaseException], ...]
+        ] = BaseException,
+    ) -> "example":
+        """Mark this example as an expected failure, like pytest.mark.xfail().
+
+        Expected-failing examples allow you to check that your test does fail on
+        some examples, and therefore build confidence that *passing* tests are
+        because your code is working, not because the test is missing something.
+
+        .. code-block:: python
+
+            @example(...).xfail()
+            @example(...).xfail(reason="Prices must be non-negative")
+            @example(...).xfail(raises=(KeyError, ValueError))
+            @example(...).xfail(sys.version_info[:2] >= (3, 9), reason="needs py39+")
+            @example(...).xfail(condition=sys.platform != "linux", raises=OSError)
+            def test(x):
+                pass
+        """
+        check_type(bool, condition, "condition")
+        check_type(str, reason, "reason")
+        if not (
+            isinstance(raises, type) and issubclass(raises, BaseException)
+        ) and not (
+            isinstance(raises, tuple)
+            and raises  # () -> expected to fail with no error, which is impossible
+            and all(
+                isinstance(r, type) and issubclass(r, BaseException) for r in raises
+            )
+        ):
+            raise InvalidArgument(
+                f"raises={raises!r} must be an exception type or tuple of exception types"
+            )
+        if condition:
+            self._this_example = attr.evolve(
+                self._this_example, raises=raises, reason=reason
+            )
+        return self
+
+    def via(self, *whence: str) -> "example":
+        """Attach a machine-readable label noting whence this example came.
+
+        The idea is that tools will be able to add ``@example()`` cases for you, e.g.
+        to maintain a high-coverage set of explicit examples, but also *remove* them
+        if they become redundant - without ever deleting manually-added examples:
+
+        .. code-block:: python
+
+            # You can choose to annotate examples, or not, as you prefer
+            @example(...)
+            @example(...).via("regression test for issue #42")
+
+            # The `hy-` prefix is reserved for automated tooling
+            @example(...).via("hy-failing")
+            @example(...).via("hy-coverage")
+            @example(...).via("hy-target-$label")
+            def test(x):
+                pass
+
+        Note that this "method chaining" syntax requires Python 3.9 or later, for
+        :pep:`614` relaxing grammar restrictions on decorators.  If you need to
+        support older versions of Python, you can use an identity function:
+
+        .. code-block:: python
+
+            def identity(x):
+                return x
+
+
+            @identity(example(...).via("label"))
+            def test(x):
+                pass
+
+        """
+        if len(whence) != 1 or not isinstance(whence[0], str):
+            raise InvalidArgument(".via() must be passed a string")
+        # This is deliberately a no-op at runtime; the tools operate on source code.
+        return self
+
+    if sys.version_info[:2] >= (3, 8):
+        # We want a positional-only argument, and on Python 3.8+ we'll get it.
+        __sig = get_signature(via)
+        via = define_function_signature(
+            name=via.__name__,
+            docstring=via.__doc__,
+            signature=__sig.replace(
+                parameters=[
+                    p.replace(kind=inspect.Parameter.POSITIONAL_ONLY)
+                    for p in __sig.parameters.values()
+                ]
+            ),
+        )(via)
+        del __sig
 
 
 def seed(seed: Hashable) -> Callable[[TestFunc], TestFunc]:
@@ -235,20 +341,6 @@ def decode_failure(blob):
         )
 
 
-class WithRunner(MappedSearchStrategy):
-    def __init__(self, base, runner):
-        assert runner is not None
-        super().__init__(base)
-        self.runner = runner
-
-    def do_draw(self, data):
-        data.hypothesis_runner = self.runner
-        return self.mapped_strategy.do_draw(data)
-
-    def __repr__(self):
-        return f"WithRunner({self.mapped_strategy!r}, runner={self.runner!r})"
-
-
 def _invalid(message, *, exc=InvalidArgument, test, given_kwargs):
     @impersonate(test)
     def wrapped_test(*arguments, **kwargs):  # pragma: no cover  # coverage limitation
@@ -292,9 +384,9 @@ def is_invalid_test(test, original_sig, given_arguments, given_kwargs):
             f"arguments, but got {len(given_arguments)} {given_arguments!r}"
         )
 
-    if infer in given_arguments:
+    if ... in given_arguments:
         return invalid(
-            "... was passed as a positional argument to @given,  but may only be "
+            "... was passed as a positional argument to @given, but may only be "
             "passed as a keyword argument or as the sole argument of @given"
         )
 
@@ -325,34 +417,8 @@ def is_invalid_test(test, original_sig, given_arguments, given_kwargs):
         )
 
 
-class ArtificialDataForExample(ConjectureData):
-    """Dummy object that pretends to be a ConjectureData object for the purposes of
-    drawing arguments for @example. Provides just enough of the ConjectureData API
-    to allow the test to run. Does not support any sort of interactive drawing,
-    but that's fine because you can't access that when all of your arguments are
-    provided by @example.
-    """
-
-    def __init__(self, args, kwargs):
-        self.__draws = 0
-        self.__args = args
-        self.__kwargs = kwargs
-        super().__init__(max_length=0, prefix=b"", random=None)
-
-    def draw_bits(self, n):
-        raise NotImplementedError("Dummy object should never be asked for bits.")
-
-    def draw(self, strategy):
-        assert self.__draws == 0
-        self.__draws += 1
-        # The main strategy for given is always a tuples strategy that returns
-        # first positional arguments then keyword arguments. When building this
-        # object already converted all positional arguments to keyword arguments,
-        # so this is the correct format to return.
-        return self.__args, self.__kwargs
-
-
 def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_sig):
+    assert isinstance(state, StateForActualGivenExecution)
     posargs = [
         p.name
         for p in original_sig.parameters.values()
@@ -360,6 +426,7 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
     ]
 
     for example in reversed(getattr(wrapped_test, "hypothesis_explicit_examples", ())):
+        assert isinstance(example, Example)
         # All of this validation is to check that @example() got "the same" arguments
         # as @given, i.e. corresponding to the same parameters, even though they might
         # be any mixture of positional and keyword arguments.
@@ -401,12 +468,53 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
         with local_settings(state.settings):
             fragments_reported = []
             try:
+                bits = ", ".join(nicerepr(x) for x in arguments) + ", ".join(
+                    f"{k}={nicerepr(v)}" for k, v in example_kwargs.items()
+                )
+                execute_example = partial(
+                    state.execute_once,
+                    ConjectureData.for_buffer(b""),
+                    is_final=True,
+                    print_example=True,
+                    example_kwargs=example_kwargs,
+                )
                 with with_reporter(fragments_reported.append):
-                    state.execute_once(
-                        ArtificialDataForExample(arguments, example_kwargs),
-                        is_final=True,
-                        print_example=True,
-                    )
+                    if example.raises is None:
+                        execute_example()
+                    else:
+                        # @example(...).xfail(...)
+                        try:
+                            execute_example()
+                        except failure_exceptions_to_catch() as err:
+                            if not isinstance(err, example.raises):
+                                raise
+                        except example.raises as err:
+                            # We'd usually check this as early as possible, but it's
+                            # possible for failure_exceptions_to_catch() to grow when
+                            # e.g. pytest is imported between import- and test-time.
+                            raise InvalidArgument(
+                                f"@example({bits}) raised an expected {err!r}, "
+                                "but Hypothesis does not treat this as a test failure"
+                            ) from err
+                        else:
+                            # Unexpectedly passing; always raise an error in this case.
+                            reason = f" because {example.reason}" * bool(example.reason)
+                            if example.raises is BaseException:
+                                name = "exception"  # special-case no raises= arg
+                            elif not isinstance(example.raises, tuple):
+                                name = example.raises.__name__
+                            elif len(example.raises) == 1:
+                                name = example.raises[0].__name__
+                            else:
+                                name = (
+                                    ", ".join(ex.__name__ for ex in example.raises[:-1])
+                                    + f", or {example.raises[-1].__name__}"
+                                )
+                            vowel = name.upper()[0] in "AEIOU"
+                            raise AssertionError(
+                                f"Expected a{'n' * vowel} {name} from @example({bits})"
+                                f"{reason}, but no exception was raised."
+                            )
             except UnsatisfiedAssumption:
                 # Odd though it seems, we deliberately support explicit examples that
                 # are then rejected by a call to `assume()`.  As well as iterative
@@ -424,7 +532,7 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
                 # One user error - whether misunderstanding or typo - we've seen a few
                 # times is to pass strategies to @example() where values are expected.
                 # Checking is easy, and false-positives not much of a problem, so:
-                if any(
+                if isinstance(err, failure_exceptions_to_catch()) and any(
                     isinstance(arg, SearchStrategy)
                     for arg in example.args + tuple(example.kwargs.values())
                 ):
@@ -440,6 +548,7 @@ def execute_explicit_examples(state, wrapped_test, arguments, kwargs, original_s
                 if (
                     state.settings.report_multiple_bugs
                     and pytest_shows_exceptiongroups
+                    and isinstance(err, failure_exceptions_to_catch())
                     and not isinstance(err, skip_exceptions_to_reraise())
                 ):
                     continue
@@ -476,6 +585,14 @@ def get_random_for_wrapped_test(test, wrapped_test):
         return Random(seed)
 
 
+@attr.s
+class Stuff:
+    selfy = attr.ib(default=None)
+    args = attr.ib(factory=tuple)
+    kwargs = attr.ib(factory=dict)
+    given_kwargs = attr.ib(factory=dict)
+
+
 def process_arguments_to_given(wrapped_test, arguments, kwargs, given_kwargs, params):
     selfy = None
     arguments, kwargs = convert_positional_arguments(wrapped_test, arguments, kwargs)
@@ -498,22 +615,14 @@ def process_arguments_to_given(wrapped_test, arguments, kwargs, given_kwargs, pa
 
     arguments = tuple(arguments)
 
-    # We use TupleStrategy over tuples() here to avoid polluting
-    # st.STRATEGY_CACHE with references (see #493), and because this is
-    # trivial anyway if the fixed_dictionaries strategy is cacheable.
-    search_strategy = TupleStrategy(
-        (
-            st.just(arguments),
-            st.fixed_dictionaries(given_kwargs).map(lambda args: dict(args, **kwargs)),
-        )
-    )
+    with ensure_free_stackframes():
+        for k, s in given_kwargs.items():
+            check_strategy(s, name=k)
+            s.validate()
 
-    if selfy is not None:
-        search_strategy = WithRunner(search_strategy, selfy)
+    stuff = Stuff(selfy=selfy, args=arguments, kwargs=kwargs, given_kwargs=given_kwargs)
 
-    search_strategy.validate()
-
-    return arguments, kwargs, test_runner, search_strategy
+    return arguments, kwargs, test_runner, stuff
 
 
 def skip_exceptions_to_reraise():
@@ -572,11 +681,9 @@ def new_given_signature(original_sig, given_kwargs):
 
 
 class StateForActualGivenExecution:
-    def __init__(
-        self, test_runner, search_strategy, test, settings, random, wrapped_test
-    ):
+    def __init__(self, test_runner, stuff, test, settings, random, wrapped_test):
         self.test_runner = test_runner
-        self.search_strategy = search_strategy
+        self.stuff = stuff
         self.settings = settings
         self.last_exception = None
         self.falsifying_examples = ()
@@ -600,7 +707,12 @@ class StateForActualGivenExecution:
         self.explain_traces = defaultdict(set)
 
     def execute_once(
-        self, data, print_example=False, is_final=False, expected_failure=None
+        self,
+        data,
+        print_example=False,
+        is_final=False,
+        expected_failure=None,
+        example_kwargs=None,
     ):
         """Run the test function once, using ``data`` as input.
 
@@ -644,18 +756,27 @@ class StateForActualGivenExecution:
             # Set up dynamic context needed by a single test run.
             with local_settings(self.settings):
                 with deterministic_PRNG():
-                    with BuildContext(data, is_final=is_final):
-
+                    with BuildContext(data, is_final=is_final) as context:
+                        if self.stuff.selfy is not None:
+                            data.hypothesis_runner = self.stuff.selfy
                         # Generate all arguments to the test function.
-                        args, kwargs = data.draw(self.search_strategy)
+                        args = self.stuff.args
+                        kwargs = dict(self.stuff.kwargs)
+                        if example_kwargs is None:
+                            a, kw, argslices = context.prep_args_kwargs_from_strategies(
+                                (), self.stuff.given_kwargs
+                            )
+                            assert not a, "strategies all moved to kwargs by now"
+                        else:
+                            kw = example_kwargs
+                            argslices = {}
+                        kwargs.update(kw)
                         if expected_failure is not None:
                             nonlocal text_repr
                             text_repr = repr_call(test, args, kwargs)
 
                         if print_example or current_verbosity() >= Verbosity.verbose:
-                            output = StringIO()
-
-                            printer = RepresentationPrinter(output)
+                            printer = RepresentationPrinter(context=context)
                             if print_example:
                                 printer.text("Falsifying example:")
                             else:
@@ -663,28 +784,19 @@ class StateForActualGivenExecution:
 
                             if self.print_given_args:
                                 printer.text(" ")
-                                printer.text(test.__name__)
-                                with printer.group(indent=4, open="(", close=""):
-                                    printer.break_()
-                                    for v in args:
-                                        printer.pretty(v)
-                                        # We add a comma unconditionally because
-                                        # generated arguments will always be kwargs,
-                                        # so there will always be more to come.
-                                        printer.text(",")
-                                        printer.breakable()
-
-                                    for i, (k, v) in enumerate(kwargs.items()):
-                                        printer.text(k)
-                                        printer.text("=")
-                                        printer.pretty(v)
-                                        printer.text(",")
-                                        if i + 1 < len(kwargs):
-                                            printer.breakable()
-                                printer.break_()
-                                printer.text(")")
-                            printer.flush()
-                            report(output.getvalue())
+                                printer.repr_call(
+                                    test.__name__,
+                                    args,
+                                    kwargs,
+                                    force_split=True,
+                                    arg_slices=argslices,
+                                    leading_comment=(
+                                        "# " + context.data.slice_comments[(0, 0)]
+                                        if (0, 0) in context.data.slice_comments
+                                        else None
+                                    ),
+                                )
+                            report(printer.getvalue())
                         return test(*args, **kwargs)
 
         # Run the test function once, via the executor hook.
@@ -864,6 +976,7 @@ class StateForActualGivenExecution:
             fragments = []
 
             ran_example = ConjectureData.for_buffer(falsifying_example.buffer)
+            ran_example.slice_comments = falsifying_example.slice_comments
             assert info.__expected_exception is not None
             try:
                 with with_reporter(fragments.append):
@@ -889,7 +1002,9 @@ class StateForActualGivenExecution:
                 errors_to_report.append(
                     (fragments, e.with_traceback(get_trimmed_traceback()))
                 )
-
+            else:
+                # execute_once() will always raise either the expected error, or Flaky.
+                raise NotImplementedError("This should be unreachable")
             finally:
                 # Whether or not replay actually raised the exception again, we want
                 # to print the reproduce_failure decorator for the failing example.
@@ -902,7 +1017,8 @@ class StateForActualGivenExecution:
                 # Mostly useful for ``find`` and ensuring that objects that
                 # hold on to a reference to ``data`` know that it's now been
                 # finished and they can't draw more data from it.
-                ran_example.freeze()
+                ran_example.freeze()  # pragma: no branch
+                # No branch is possible here because we never have an active exception.
         _raise_to_user(errors_to_report, self.settings, report_lines)
 
 
@@ -917,10 +1033,15 @@ def add_note(exc, note):
 
 def _raise_to_user(errors_to_report, settings, target_lines, trailer=""):
     """Helper function for attaching notes and grouping multiple errors."""
-    if settings.verbosity >= Verbosity.normal:
-        for fragments, err in errors_to_report:
-            for note in fragments:
-                add_note(err, note)
+    failing_prefix = "Falsifying example: "
+    ls = []
+    for fragments, err in errors_to_report:
+        for note in fragments:
+            add_note(err, note)
+            if note.startswith(failing_prefix):
+                ls.append(note[len(failing_prefix) :])
+    if current_pytest_item.value:
+        current_pytest_item.value._hypothesis_failing_examples = ls
 
     if len(errors_to_report) == 1:
         _, the_error_hypothesis_found = errors_to_report[0]
@@ -998,7 +1119,7 @@ class HypothesisHandle:
 
 @overload
 def given(
-    *_given_arguments: Union[SearchStrategy[Any], InferType],
+    *_given_arguments: Union[SearchStrategy[Any], EllipsisType],
 ) -> Callable[
     [Callable[..., Optional[Coroutine[Any, Any, None]]]], Callable[..., None]
 ]:  # pragma: no cover
@@ -1007,7 +1128,7 @@ def given(
 
 @overload
 def given(
-    **_given_kwargs: Union[SearchStrategy[Any], InferType],
+    **_given_kwargs: Union[SearchStrategy[Any], EllipsisType],
 ) -> Callable[
     [Callable[..., Optional[Coroutine[Any, Any, None]]]], Callable[..., None]
 ]:  # pragma: no cover
@@ -1015,8 +1136,8 @@ def given(
 
 
 def given(
-    *_given_arguments: Union[SearchStrategy[Any], InferType],
-    **_given_kwargs: Union[SearchStrategy[Any], InferType],
+    *_given_arguments: Union[SearchStrategy[Any], EllipsisType],
+    **_given_kwargs: Union[SearchStrategy[Any], EllipsisType],
 ) -> Callable[
     [Callable[..., Optional[Coroutine[Any, Any, None]]]], Callable[..., None]
 ]:
@@ -1062,16 +1183,16 @@ def given(
                 for p in original_sig.parameters.values()
                 if p.kind is p.POSITIONAL_OR_KEYWORD
             ]
-            given_kwargs = dict(zip(posargs[::-1], given_arguments[::-1]))
+            given_kwargs = dict(list(zip(posargs[::-1], given_arguments[::-1]))[::-1])
         # These have been converted, so delete them to prevent accidental use.
         del given_arguments
 
         new_signature = new_given_signature(original_sig, given_kwargs)
 
         # Use type information to convert "infer" arguments into appropriate strategies.
-        if infer in given_kwargs.values():
+        if ... in given_kwargs.values():
             hints = get_type_hints(test)
-        for name in [name for name, value in given_kwargs.items() if value is infer]:
+        for name in [name for name, value in given_kwargs.items() if value is ...]:
             if name not in hints:
                 return _invalid(
                     f"passed {name}=... for {test.__name__}, but {name} has "
@@ -1106,7 +1227,7 @@ def given(
             processed_args = process_arguments_to_given(
                 wrapped_test, arguments, kwargs, given_kwargs, new_signature.parameters
             )
-            arguments, kwargs, test_runner, search_strategy = processed_args
+            arguments, kwargs, test_runner, stuff = processed_args
 
             if (
                 inspect.iscoroutinefunction(test)
@@ -1124,8 +1245,8 @@ def given(
                     "readthedocs.io/en/latest/details.html#custom-function-execution"
                 )
 
-            runner = getattr(search_strategy, "runner", None)
-            if isinstance(runner, TestCase) and test.__name__ in dir(TestCase):
+            runner = stuff.selfy
+            if isinstance(stuff.selfy, TestCase) and test.__name__ in dir(TestCase):
                 msg = (
                     f"You have applied @given to the method {test.__name__}, which is "
                     "used by the unittest runner but is not itself a test."
@@ -1144,7 +1265,7 @@ def given(
                 )
 
             state = StateForActualGivenExecution(
-                test_runner, search_strategy, test, settings, random, wrapped_test
+                test_runner, stuff, test, settings, random, wrapped_test
             )
 
             reproduce_failure = wrapped_test._hypothesis_internal_use_reproduce_failure
@@ -1270,9 +1391,9 @@ def given(
             if not (ran_explicit_examples or state.ever_executed):
                 raise SKIP_BECAUSE_NO_EXAMPLES
 
-        def _get_fuzz_target() -> Callable[
-            [Union[bytes, bytearray, memoryview, BinaryIO]], Optional[bytes]
-        ]:
+        def _get_fuzz_target() -> (
+            Callable[[Union[bytes, bytearray, memoryview, BinaryIO]], Optional[bytes]]
+        ):
             # Because fuzzing interfaces are very performance-sensitive, we use a
             # somewhat more complicated structure here.  `_get_fuzz_target()` is
             # called by the `HypothesisHandle.fuzz_one_input` property, allowing
@@ -1287,13 +1408,13 @@ def given(
                 parent=wrapped_test._hypothesis_internal_use_settings, deadline=None
             )
             random = get_random_for_wrapped_test(test, wrapped_test)
-            _args, _kwargs, test_runner, search_strategy = process_arguments_to_given(
+            _args, _kwargs, test_runner, stuff = process_arguments_to_given(
                 wrapped_test, (), {}, given_kwargs, new_signature.parameters
             )
             assert not _args
             assert not _kwargs
             state = StateForActualGivenExecution(
-                test_runner, search_strategy, test, settings, random, wrapped_test
+                test_runner, stuff, test, settings, random, wrapped_test
             )
             digest = function_digest(test)
             # We track the minimal-so-far example for each distinct origin, so
@@ -1308,7 +1429,7 @@ def given(
                 # This inner part is all that the fuzzer will actually run,
                 # so we keep it as small and as fast as possible.
                 if isinstance(buffer, io.IOBase):
-                    buffer = buffer.read()
+                    buffer = buffer.read(BUFFER_SIZE)
                 assert isinstance(buffer, (bytes, bytearray, memoryview))
                 data = ConjectureData.for_buffer(buffer)
                 try:
@@ -1367,7 +1488,7 @@ def find(
     if settings is None:
         settings = Settings(max_examples=2000)
     settings = Settings(
-        settings, suppress_health_check=HealthCheck.all(), report_multiple_bugs=False
+        settings, suppress_health_check=list(HealthCheck), report_multiple_bugs=False
     )
 
     if database_key is None and settings.database is not None:
