@@ -15,10 +15,15 @@ import math
 import sys
 from collections import OrderedDict, abc
 from functools import lru_cache
+from typing import TYPE_CHECKING, List, Optional, Sequence, Tuple, Type, TypeVar, Union
 
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import floor, int_from_bytes
-from hypothesis.internal.floats import int_to_float
+from hypothesis.internal.floats import int_to_float, next_up
+
+if TYPE_CHECKING:
+    from hypothesis.internal.conjecture.data import ConjectureData
+
 
 LABEL_MASK = 2**64 - 1
 
@@ -47,7 +52,7 @@ SAMPLE_IN_SAMPLER_LABEL = calc_label_from_name("a sample() in Sampler")
 ONE_FROM_MANY_LABEL = calc_label_from_name("one more from many()")
 
 
-def unbounded_integers(data):
+def unbounded_integers(data: "ConjectureData") -> int:
     size = INT_SIZES[INT_SIZES_SAMPLER.sample(data)]
     r = data.draw_bits(size)
     sign = r & 1
@@ -57,8 +62,15 @@ def unbounded_integers(data):
     return int(r)
 
 
-def integer_range(data, lower, upper, center=None):
+def integer_range(
+    data: "ConjectureData",
+    lower: int,
+    upper: int,
+    center: Optional[int] = None,
+    forced: Optional[int] = None,
+) -> int:
     assert lower <= upper
+    assert forced is None or lower <= forced <= upper
     if lower == upper:
         # Write a value even when this is trivial so that when a bound depends
         # on other values we don't suddenly disappear when the gap shrinks to
@@ -76,7 +88,8 @@ def integer_range(data, lower, upper, center=None):
     elif center == lower:
         above = True
     else:
-        above = not boolean(data)
+        force_above = None if forced is None else forced < center
+        above = not data.draw_bits(1, forced=force_above)
 
     if above:
         gap = upper - center
@@ -88,7 +101,7 @@ def integer_range(data, lower, upper, center=None):
     bits = gap.bit_length()
     probe = gap + 1
 
-    if bits > 24 and data.draw_bits(3):
+    if bits > 24 and data.draw_bits(3, forced=None if forced is None else 0):
         # For large ranges, we combine the uniform random distribution from draw_bits
         # with a weighting scheme with moderate chance.  Cutoff at 2 ** 24 so that our
         # choice of unicode characters is uniform but the 32bit distribution is not.
@@ -97,7 +110,9 @@ def integer_range(data, lower, upper, center=None):
 
     while probe > gap:
         data.start_example(INTEGER_RANGE_DRAW_LABEL)
-        probe = data.draw_bits(bits)
+        probe = data.draw_bits(
+            bits, forced=None if forced is None else abs(forced - center)
+        )
         data.stop_example(discard=probe > gap)
 
     if above:
@@ -106,10 +121,16 @@ def integer_range(data, lower, upper, center=None):
         result = center - probe
 
     assert lower <= result <= upper
-    return int(result)
+    assert forced is None or result == forced, (result, forced, center, above)
+    return result
 
 
-def check_sample(values, strategy_name):
+T = TypeVar("T")
+
+
+def check_sample(
+    values: Union[Type[enum.Enum], Sequence[T]], strategy_name: str
+) -> Sequence[T]:
     if "numpy" in sys.modules and isinstance(values, sys.modules["numpy"].ndarray):
         if values.ndim != 1:
             raise InvalidArgument(
@@ -127,7 +148,7 @@ def check_sample(values, strategy_name):
             "strategy has stable results between runs. To replay a saved "
             "example, the sampled values must have the same iteration order "
             "on every run - ruling out sets, dicts, etc due to hash "
-            "randomisation. Most cases can simply use `sorted(values)`, but "
+            "randomization. Most cases can simply use `sorted(values)`, but "
             "mixed types or special values such as math.nan require careful "
             "handling - and note that when simplifying an example, "
             "Hypothesis treats earlier values as simpler."
@@ -137,7 +158,7 @@ def check_sample(values, strategy_name):
     return tuple(values)
 
 
-def choice(data, values):
+def choice(data: "ConjectureData", values: Sequence[T]) -> T:
     return values[integer_range(data, 0, len(values) - 1)]
 
 
@@ -145,15 +166,13 @@ FLOAT_PREFIX = 0b1111111111 << 52
 FULL_FLOAT = int_to_float(FLOAT_PREFIX | ((2 << 53) - 1)) - 1
 
 
-def fractional_float(data):
+def fractional_float(data: "ConjectureData") -> float:
     return (int_to_float(FLOAT_PREFIX | data.draw_bits(52)) - 1) / FULL_FLOAT
 
 
-def boolean(data):
-    return bool(data.draw_bits(1))
-
-
-def biased_coin(data, p, *, forced=None):
+def biased_coin(
+    data: "ConjectureData", p: float, *, forced: Optional[bool] = None
+) -> bool:
     """Return True with probability p (assuming a uniform generator),
     shrinking towards False. If ``forced`` is set to a non-None value, this
     will always return that value but will write choices appropriate to having
@@ -188,7 +207,7 @@ def biased_coin(data, p, *, forced=None):
         # The logic here is a bit complicated and special cased to make it
         # play better with the shrinker.
 
-        # We imagine partitioning the real interval [0, 1] into 256 equal parts
+        # We imagine partitioning the real interval [0, 1] into 2**n equal parts
         # and looking at each part and whether its interior is wholly <= p
         # or wholly >= p. At most one part can be neither.
 
@@ -232,9 +251,9 @@ def biased_coin(data, p, *, forced=None):
             else:
                 i = data.draw_bits(bits, forced=int(forced))
 
-            # We always label the region that causes us to repeat the loop as
-            # 255 so that shrinking this byte never causes us to need to draw
-            # more data.
+            # We always choose the region that causes us to repeat the loop as
+            # the maximum value, so that shrinking the drawn bits never causes
+            # us to need to draw more data.
             if partial and i == size - 1:
                 p = remainder
                 continue
@@ -289,30 +308,31 @@ class Sampler:
        shrinking the chosen element.
     """
 
-    def __init__(self, weights):
+    table: List[Tuple[int, int, float]]  # (base_idx, alt_idx, alt_chance)
 
+    def __init__(self, weights: Sequence[float]):
         n = len(weights)
 
-        self.table = [[i, None, None] for i in range(n)]
+        table: "list[list[int | float | None]]" = [[i, None, None] for i in range(n)]
 
         total = sum(weights)
 
         num_type = type(total)
 
-        zero = num_type(0)
-        one = num_type(1)
+        zero = num_type(0)  # type: ignore
+        one = num_type(1)  # type: ignore
 
-        small = []
-        large = []
+        small: "List[int]" = []
+        large: "List[int]" = []
 
         probabilities = [w / total for w in weights]
-        scaled_probabilities = []
+        scaled_probabilities: "List[float]" = []
 
-        for i, p in enumerate(probabilities):
-            scaled = p * n
+        for i, alternate_chance in enumerate(probabilities):
+            scaled = alternate_chance * n
             scaled_probabilities.append(scaled)
             if scaled == 1:
-                self.table[i][2] = zero
+                table[i][2] = zero
             elif scaled < 1:
                 small.append(i)
             else:
@@ -326,9 +346,9 @@ class Sampler:
 
             assert lo != hi
             assert scaled_probabilities[hi] > one
-            assert self.table[lo][1] is None
-            self.table[lo][1] = hi
-            self.table[lo][2] = one - scaled_probabilities[lo]
+            assert table[lo][1] is None
+            table[lo][1] = hi
+            table[lo][2] = one - scaled_probabilities[lo]
             scaled_probabilities[hi] = (
                 scaled_probabilities[hi] + scaled_probabilities[lo]
             ) - one
@@ -336,27 +356,29 @@ class Sampler:
             if scaled_probabilities[hi] < 1:
                 heapq.heappush(small, hi)
             elif scaled_probabilities[hi] == 1:
-                self.table[hi][2] = zero
+                table[hi][2] = zero
             else:
                 heapq.heappush(large, hi)
         while large:
-            self.table[large.pop()][2] = zero
+            table[large.pop()][2] = zero
         while small:
-            self.table[small.pop()][2] = zero
+            table[small.pop()][2] = zero
 
-        for entry in self.table:
-            assert entry[2] is not None
-            if entry[1] is None:
-                entry[1] = entry[0]
-            elif entry[1] < entry[0]:
-                entry[0], entry[1] = entry[1], entry[0]
-                entry[2] = one - entry[2]
+        self.table: "List[Tuple[int, int, float]]" = []
+        for base, alternate, alternate_chance in table:  # type: ignore
+            assert isinstance(base, int)
+            assert isinstance(alternate, int) or alternate is None
+            if alternate is None:
+                self.table.append((base, base, alternate_chance))
+            elif alternate < base:
+                self.table.append((alternate, base, one - alternate_chance))
+            else:
+                self.table.append((base, alternate, alternate_chance))
         self.table.sort()
 
-    def sample(self, data):
+    def sample(self, data: "ConjectureData") -> int:
         data.start_example(SAMPLE_IN_SAMPLER_LABEL)
-        i = integer_range(data, 0, len(self.table) - 1)
-        base, alternate, alternate_chance = self.table[i]
+        base, alternate, alternate_chance = choice(data, self.table)
         use_alternate = biased_coin(data, alternate_chance)
         data.stop_example()
         if use_alternate:
@@ -381,7 +403,13 @@ class many:
         add_stuff_to_result()
     """
 
-    def __init__(self, data, min_size, max_size, average_size):
+    def __init__(
+        self,
+        data: "ConjectureData",
+        min_size: int,
+        max_size: Union[int, float],
+        average_size: Union[int, float],
+    ) -> None:
         assert 0 <= min_size <= average_size <= max_size
         self.min_size = min_size
         self.max_size = max_size
@@ -393,7 +421,7 @@ class many:
         self.force_stop = False
         self.rejected = False
 
-    def more(self):
+    def more(self) -> bool:
         """Should I draw another element to add to the collection?"""
         if self.drawn:
             self.data.stop_example(discard=self.rejected)
@@ -424,7 +452,7 @@ class many:
             self.data.stop_example()
             return False
 
-    def reject(self):
+    def reject(self, why: Optional[str] = None) -> None:
         """Reject the last example (i.e. don't count it towards our budget of
         elements because it's not going to go in the final collection)."""
         assert self.count > 0
@@ -435,42 +463,54 @@ class many:
         # failing too fast when we reject the first draw.
         if self.rejections > max(3, 2 * self.count):
             if self.count < self.min_size:
-                self.data.mark_invalid()
+                self.data.mark_invalid(why)
             else:
                 self.force_stop = True
 
 
+SMALLEST_POSITIVE_FLOAT: float = next_up(0.0) or sys.float_info.min
+
+
 @lru_cache()
-def _calc_p_continue(desired_avg, max_size):
+def _calc_p_continue(desired_avg: float, max_size: int) -> float:
     """Return the p_continue which will generate the desired average size."""
+    assert desired_avg <= max_size, (desired_avg, max_size)
     if desired_avg == max_size:
         return 1.0
     p_continue = 1 - 1.0 / (1 + desired_avg)
     if p_continue == 0 or max_size == float("inf"):
         assert 0 <= p_continue < 1, p_continue
         return p_continue
+    assert 0 < p_continue < 1, p_continue
     # For small max_size, the infinite-series p_continue is a poor approximation,
     # and while we can't solve the polynomial a few rounds of iteration quickly
     # gets us a good approximate solution in almost all cases (sometimes exact!).
     while _p_continue_to_avg(p_continue, max_size) > desired_avg:
         # This is impossible over the reals, but *can* happen with floats.
         p_continue -= 0.0001
+        # If we've reached zero or gone negative, we want to break out of this loop,
+        # and do so even if we're on a system with the unsafe denormals-are-zero flag.
+        # We make that an explicit error in st.floats(), but here we'd prefer to
+        # just get somewhat worse precision on collection lengths.
+        if p_continue < SMALLEST_POSITIVE_FLOAT:
+            p_continue = SMALLEST_POSITIVE_FLOAT
+            break
     # Let's binary-search our way to a better estimate!  We tried fancier options
     # like gradient descent, but this is numerically stable and works better.
     hi = 1.0
     while desired_avg - _p_continue_to_avg(p_continue, max_size) > 0.01:
-        assert p_continue < hi
+        assert 0 < p_continue < hi, (p_continue, hi)
         mid = (p_continue + hi) / 2
         if _p_continue_to_avg(mid, max_size) <= desired_avg:
             p_continue = mid
         else:
             hi = mid
-    assert 0 < p_continue < 1
+    assert 0 < p_continue < 1, p_continue
     assert _p_continue_to_avg(p_continue, max_size) <= desired_avg
     return p_continue
 
 
-def _p_continue_to_avg(p_continue, max_size):
+def _p_continue_to_avg(p_continue: float, max_size: int) -> float:
     """Return the average_size generated by this p_continue and max_size."""
     if p_continue >= 1:
         return max_size
